# -*- coding: utf-8 -*-

import os
import sys
import json


project_name = sys.argv[1:][0]
base_dir = sys.argv[1:][1]
noconf = sys.argv[1:][2]

filepath = base_dir + '/centrifugo/config.json'
filepathtmp = base_dir + '/tmp.py'
with open(filepath) as f:
    content = f.read()
f = open(filepath, "r")
f2 = open(filepathtmp, "w+")
conf = json.loads(content)
conf["anonymous"] = "true"
key = conf["secret"]
res = json.dumps(conf, indent=4).replace('"true"', 'true')
f2.write(res)
f2.close()
os.remove(filepath)
os.rename(filepathtmp, filepath)
# settings.py
extralines = 'SITE_SLUG = "' + project_name + '"\n'
extralines = extralines + 'SITE_NAME = SITE_SLUG\n'
extralines = extralines + 'CENTRIFUGO_SECRET_KEY = "' + key + '"\n'
extralines = extralines + 'CENTRIFUGO_HOST = "http://localhost"\n'
extralines = extralines + "CENTRIFUGO_PORT = 8001\n"


if noconf != "noconf":
    project_dir = base_dir + "/" + project_name
    filepath = project_dir + '/settings.py'
    f = open(filepath, "a")
    f.write("\n" + extralines + "\n")
    f.close()



# -*- coding: utf-8 -*-

import json
from django.http import JsonResponse
from django.core.urlresolvers import reverse
from django.http.response import Http404
from django.views.generic.base import View
from django.views.generic import FormView
from django.views.generic.base import TemplateView
from django.views.decorators.csrf import csrf_exempt
from django.utils.translation import ugettext_lazy as _
from django.contrib import messages
from instant.producers import publish
from instant.forms import BroadcastForm
from instant.utils import signed_response
from instant.conf import USERS_CHANNELS, STAFF_CHANNELS, SUPERUSER_CHANNELS


@csrf_exempt
def instant_auth(request):
    if not request.is_ajax() or not request.method == "POST":
        raise Http404
    data = json.loads(request.body.decode("utf-8"))
    channels = data["channels"]
    client = data['client']
    response = {}
    for channel in channels:
        signature = None
        if channel in USERS_CHANNELS:
            if request.user.is_authenticated():
                signature = signed_response(channel, client)
        if channel in STAFF_CHANNELS:
            if request.user.is_staff:
                signature = signed_response(channel, client)
        if channel in SUPERUSER_CHANNELS:
            if request.user.is_superuser:
                signature = signed_response(channel, client)
        if signature is not None:
            response[channel] = signature
        else:
            response[channel] = {"status": "403"}
    return JsonResponse(response)


class StaffChannelView(TemplateView):
    template_name = 'instant/channels/staff.html'

    def dispatch(self, request, *args, **kwargs):
        if not self.request.is_ajax():
            raise Http404
        return super(StaffChannelView, self).dispatch(request, *args, **kwargs)


class FrontendView(FormView):
    form_class = BroadcastForm
    template_name = 'instant/frontend/index.html'

    def dispatch(self, request, *args, **kwargs):
        if not self.request.user.is_superuser:
            raise Http404
        return super(FrontendView, self).dispatch(request, *args, **kwargs)

    def form_valid(self, form):
        print(form.cleaned_data)

        msg = form.cleaned_data['message']
        event_class = "default"
        if "event_class" in form.cleaned_data:
            event_class = form.cleaned_data['event_class']
        channel = form.cleaned_data['channel']
        default_channel = form.cleaned_data['default_channel']
        err = None
        if channel or default_channel:
            if default_channel:
                err = publish(message=msg, event_class=event_class,
                              channel=default_channel)
            if channel:
                err = publish(
                    message=msg, event_class=event_class, channel=channel)
            if err is not None:
                messages.warning(self.request, _(
                    u"Error publishing the message: " + err))
            else:
                messages.success(self.request, _(
                    u"Message published to the channel " + channel))
        else:
            messages.warning(self.request, _(
                u"Please provide a valid channel"))
        return super(FrontendView, self).form_valid(form)

    def get_success_url(self):
        return reverse('instant-message-broadcasted')


class PostMsgView(View):

    def post(self, request, *args, **kwargs):
        if not request.user.is_superuser:
            return JsonResponse({"ok": 0})
        data = json.loads(self.request.body.decode('utf-8'))
        msg = data["msg"]
        channel = data["channel"]
        event_class = "default"
        if "event_class" in data:
            event_class = data['event_class']
        err = publish(message=msg, event_class=event_class, channel=channel)
        if err is not None:
            errmsg = _(u"Error publishing the message: " + err)
            return JsonResponse({"ok": 0, "err": errmsg})
        return JsonResponse({"ok": 1})



# -*- coding: utf-8 -*-

from cent.core import generate_channel_sign
from instant.conf import PUBLIC_CHANNEL, SECRET_KEY


def signed_response(channel, client):
    signature = generate_channel_sign(SECRET_KEY, client, channel, info="")
    return {"sign": signature}


def _get_public_channel():
    return PUBLIC_CHANNEL



# -*- coding: utf-8 -*-

from __future__ import print_function
import json
from cent import Client, CentException
from instant.conf import SITE_NAME, CENTRIFUGO_HOST, CENTRIFUGO_PORT, SECRET_KEY, SITE_SLUG, PUBLIC_CHANNEL, BROADCAST_WITH
if BROADCAST_WITH == "go":
    import os
    import instant


def _get_channel(channel, target):
    if channel is None:
        if target is not None:
            if target == "superuser":
                channel = "$" + SITE_SLUG + '_admin'
            elif target == "staff":
                channel = "$" + SITE_SLUG + '_staff'
            elif target == "users":
                channel = "$" + SITE_SLUG + '_users'
            else:
                return False, None
        else:
            channel = PUBLIC_CHANNEL
    return channel


def publish_py(message, event_class="default", data={}, channel=None, site=SITE_NAME, target=None):
    cent_url = CENTRIFUGO_HOST + ":" + str(CENTRIFUGO_PORT)
    client = Client(cent_url, SECRET_KEY, timeout=1)
    channel = _get_channel(channel, target)
    payload = {"message": message, "channel": channel,
               'event_class': event_class, "data": data, "site": site}
    err = None
    try:
        client.publish(channel, payload)
    except CentException as e:
        err = str(e)
    if event_class.lower() == "debug":
        print("[DEBUG] ", str(json.dumps(payload)))
    return err


def publish_go(message, event_class="default", data={}, channel=None, site=SITE_NAME, target=None):
    channel = _get_channel(channel, target)
    channel = channel.replace("$", "-_-")
    conn = '-host="' + CENTRIFUGO_HOST + '" -port="' + \
        str(CENTRIFUGO_PORT) + '" -key="' + SECRET_KEY + '"'
    params = conn + ' -channel="' + channel + '" -event_class="' + event_class + \
        '" -message="' + message + '" -data=\'' + json.dumps(data) + "'"
    pth = os.path.dirname(instant.__file__)
    gocmd = pth + '/go/publish ' + params
    os.system(gocmd)
    if event_class.lower() == "debug":
        print("[DEBUG] ", message, str(json.dumps(data)))
    return


if BROADCAST_WITH == "go":
    publish = publish_go
else:
    publish = publish_py



# -*- coding: utf-8 -*-

from django.core.urlresolvers import reverse_lazy
from django.conf.urls import url
from django.views.generic import RedirectView
from instant.views import FrontendView, StaffChannelView, PostMsgView


urlpatterns = [
    url(r'^redirect-home/$', RedirectView.as_view(url=reverse_lazy('instant-broadcast')),
        name='instant-message-broadcasted'),
    url(r'^staff/$', StaffChannelView.as_view(), name="instant-staff-channel"),
    url(r'^rest/$', PostMsgView.as_view(), name="instant-post-msg"),
    url(r'^', FrontendView.as_view(), name="instant-broadcast")
]



from setuptools import setup, find_packages


version = __import__('instant').__version__

setup(
    name = 'django-instant',
    packages=find_packages(),
    include_package_data=True,
    version = version,
    description = 'Websockets for Django with Centrifugo ',
    author = 'synw',
    author_email = 'synwe@yahoo.com',
    url = 'https://github.com/synw/django-instant', 
    download_url = 'https://github.com/synw/django-instant/releases/tag/'+version, 
    keywords = ['django', 'websockets', 'centrifugo'], 
    classifiers = [
          'Development Status :: 4 - Beta',
          'Framework :: Django :: 1.11',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.5',
      ],
    install_requires=[
        'cent',
    ],
    zip_safe=False
)



from __future__ import print_function
from django.core.management.base import BaseCommand
import os
import subprocess
from django.conf import settings


class Command(BaseCommand):
    help = 'Install the Centrifugo websockets server for Linux'

    def handle(self, *args, **options):
        basepath = os.getcwd()
        path = os.path.abspath(__file__)
        modpath = os.path.dirname(path)
        project_name = settings.BASE_DIR.split("/")[-1:][0]
        script = modpath + "/install.sh"
        subprocess.call([script, project_name, basepath, modpath])
        return



# -*- coding: utf-8 -*-

from django.conf import settings


SECRET_KEY = getattr(settings, 'CENTRIFUGO_SECRET_KEY', "")
CENTRIFUGO_HOST = getattr(settings, 'CENTRIFUGO_HOST', 'http://localhost')
CENTRIFUGO_PORT = getattr(settings, 'CENTRIFUGO_PORT', 8001)

SITE_SLUG = getattr(settings, 'SITE_SLUG', 'site')
SITE_NAME = getattr(settings, 'SITE_NAME', 'Site')

BROADCAST_WITH = getattr(settings, 'INSTANT_BROADCAST_WITH', "py")

ENABLE_USERS_CHANNEL = getattr(settings, 'INSTANT_ENABLE_USERS_CHANNEL', False)
ENABLE_STAFF_CHANNEL = getattr(settings, 'INSTANT_ENABLE_STAFF_CHANNEL', False)
ENABLE_SUPERUSER_CHANNEL = getattr(
    settings, 'INSTANT_ENABLE_SUPERUSER_CHANNEL', False)

DEFAULT_USERS_CHANNEL = "$" + SITE_SLUG + "_users"
DEFAULT_STAFF_CHANNEL = "$" + SITE_SLUG + "_staff"
DEFAULT_SUPERUSER_CHANNEL = "$" + SITE_SLUG + "_admin"
public_channel = SITE_SLUG + '_public'
PUBLIC_CHANNEL = getattr(settings, 'INSTANT_PUBLIC_CHANNEL', public_channel)
USERS_CHANNELS = getattr(settings, 'INSTANT_USERS_CHANNELS', [
                         DEFAULT_USERS_CHANNEL])
STAFF_CHANNELS = getattr(settings, 'INSTANT_STAFF_CHANNELS', [
                         DEFAULT_STAFF_CHANNEL])
SUPERUSER_CHANNELS = getattr(settings, 'INSTANT_SUPERUSER_CHANNELS', [
                             DEFAULT_SUPERUSER_CHANNEL])

EXCLUDE = getattr(settings, 'INSTANT_EXCLUDE', ["__presence__"])

# ensure that the private channels will always be treated as private by
# Centrifugo
chans = []
for chan in USERS_CHANNELS:
    if not chan.startswith("$"):
        chan = u"$" + chan
    chans.append(chan)
USERS_CHANNELS = chans
schans = []
for chan in STAFF_CHANNELS:
    if not chan.startswith("$"):
        chan = u"$" + chan
    schans.append(chan)
STAFF_CHANNELS = schans
achans = []
for chan in SUPERUSER_CHANNELS:
    if not chan.startswith("$"):
        chan = u"$" + chan
    achans.append(chan)
SUPERUSER_CHANNELS = achans
